<!DOCTYPE html>
<html>
   <head>
      <meta charset="UTF-8">
      {% block header %}
      {% endblock %}
   </head>
   <body>
      <h1>WAMP-Klein Application</h1>
      <p>Open JavaScript console to watch output.</p>
      {% block body %}
      {% endblock %}
   </body>
</html>
